/*
 * PebblePlotter.java
 *
 * Created on June 29, 2007, 10:03 AM
 *
GeoShear - A program to explore deformation of cross sections of stone
    
Copyright (C) September 2007, Chris Warren (Chris.Warren@williams.edu) and 
Paul Karabinos (Paul.M.Karabinos@williams.edu)

This program is licensed under the terms of the Williams College Software 
License as published by Williams College; either version 1 of the License, or 
(at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
PARTICULAR PURPOSE. See the Williams College Software License for more details.

You should have received a copy of the Williams College Software License along 
with this program; if not, write to Williams College Office for Information 
Technology, 22 Lab Campus Drive, Williamstown, MA 01267 USA
 *
 */

package edu.williams.geoshear;

import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.ListIterator;
import javax.swing.JPanel;

/**
 *
 * @author cwarren
 */
public abstract class PebblePlotter extends JPanel
{

    protected ArrayList pebbleList;
    protected ArrayList chartPebbleList; // pebbles sans scaling, used as the charting data
    protected Point plotOrigin;
    
    private BufferedImage plotFrame;
    
    protected String title = "";

    protected boolean manualXMax = false;
    protected double xMaxValAuto = 0.0;

    protected boolean showTrace = false;

    private Font plotFont = Font.decode ("Arial-PLAIN-9");
    private Font plotTitleFont = Font.decode ("Arial-BOLD-11");

    private int generalInset = 4;
    private int textAllowance = 16;

    private int markSize = 1;
    private boolean markFill = false;
    private int markShape = PebblePlotter.MARK_RECT;

    private boolean showSnapshotsTrace = true;
    
    protected final static AffineTransformOp textTurnerOp = new AffineTransformOp (AffineTransform.getRotateInstance (Math.PI/2.0),AffineTransformOp.TYPE_NEAREST_NEIGHBOR);
    public static int MARK_RECT = 1;
    public static int MARK_CIRCLE = 2;

    protected boolean showMeans = false;
    protected double harmonicMean = 0;
    protected double vectorMean = 0;

    /** Creates a new instance of PebblePlotter */
    public PebblePlotter ()
    {
    }

    public abstract void setUpPlot ();
    
    protected void showMessageOn(String msg,Graphics2D g2D)
    {
        g2D.drawString(msg, 30, 30);
    }

    protected void setMeans()
    {
        double sumRFinvert = 0;
        double sumSin2Phi = 0;
        double sumCos2Phi = 0;
        
        ListIterator pli = this.getChartPebbleList().listIterator ();
        while (pli.hasNext ())
        {
            Pebble peb = (Pebble)(pli.next ());
            sumRFinvert += (1/peb.getRatioLongToShort());
            //sumSin2Phi += Math.sin(2.0*peb.getThetaRad());
            //sumCos2Phi += Math.cos(2.0*peb.getThetaRad());
            sumSin2Phi += Math.sin(peb.getThetaRad());
            sumCos2Phi += Math.cos(peb.getThetaRad());
        }
        
        this.harmonicMean = ((double)(this.getChartPebbleList().size())) / sumRFinvert;
        //this.vectorMean = .5 * Math.atan(sumSin2Phi/sumCos2Phi);        
        this.vectorMean = Math.atan(sumSin2Phi/sumCos2Phi);        
    }

    protected abstract void paintMeans(Graphics2D g2D);

    public ArrayList getPebbleList ()
    {
        return pebbleList;
    }

    public ArrayList getChartPebbleList ()
    {
        return chartPebbleList;
    }
    
    public void setPebbleList (ArrayList pebbleList)
    {
        this.pebbleList = pebbleList;
    }

    public void setChartPebbleList (ArrayList pebbleList)
    {
        this.chartPebbleList = pebbleList;
    }
    
    public Point getPlotOrigin ()
    {
        return plotOrigin;
    }
    
    public void setPlotOrigin (Point plotOrigin)
    {
        this.plotOrigin = plotOrigin;
    }

    public BufferedImage getPlotFrame ()
    {
        return plotFrame;
    }
    
    public void setPlotFrame (BufferedImage plotFrame)
    {
        this.plotFrame = plotFrame;
    }
    

    public String getTitle ()
    {
        return title;
    }
    
    public void setTitle (String title)
    {
        this.title = title;
        this.setUpPlotFrame ();
    }
    
    protected abstract void setUpPlotFrame();
    
    public boolean isManualXMax ()
    {
        return manualXMax;
    }

    public void setManualXMax (boolean manualXMax)
    {
        this.manualXMax = manualXMax;
    }

    public double getXMaxValAuto ()
    {
        return xMaxValAuto;
    }

    public void setXMaxValAuto (double xMaxValAuto)
    {
        this.xMaxValAuto = xMaxValAuto;
    }
    
    public Font getPlotFont ()
    {
        return plotFont;
    }
    
    public void setPlotFont (Font plotFont)
    {
        this.plotFont = plotFont;
        this.setUpPlot ();
    }
    
    public Font getPlotTitleFont ()
    {
        return plotTitleFont;
    }
    
    public void setPlotTitleFont (Font plotTitleFont)
    {
        this.plotTitleFont = plotTitleFont;
        this.setUpPlot ();
    }

    public int getGeneralInset ()
    {
        return generalInset;
    }

    public void setGeneralInset (int generalInset)
    {
        this.generalInset = generalInset;
    }

    public int getTextAllowance ()
    {
        return textAllowance;
    }

    public void setTextAllowance (int textAllowance)
    {
        this.textAllowance = textAllowance;
    }

    public int getMarkSize ()
    {
        return markSize;
    }

    public void setMarkSize (int markSize)
    {
        this.markSize = markSize;
    }

    public boolean isMarkFill ()
    {
        return markFill;
    }

    public void setMarkFill (boolean markFill)
    {
        this.markFill = markFill;
    }

    public int getMarkShape ()
    {
        return markShape;
    }
    
    public void setMarkShape (int markShape)
    {
        this.markShape = markShape;
    }

    public boolean isShowSnapshotsTrace() {
        return showSnapshotsTrace;
    }

    public void setShowSnapshotsTrace(boolean showSnapshotsTrace) {
        this.showSnapshotsTrace = showSnapshotsTrace;
    }

    public boolean isShowTrace() {
        return showTrace;
    }

    public void setShowTrace(boolean showTrace) {
        this.showTrace = showTrace;
    }
    
    public boolean isShowHarmonics() {
        return showMeans;
    }

    public void setShowHarmonics(boolean showHarmonics) {
        this.showMeans = showHarmonics;
    }


}
