/*
 * PebblePlotterCartesian.java
 *
 * Created on March 21, 2007, 9:59 AM
 *
GeoShear - A program to explore deformation of cross sections of stone
    
Copyright (C) September 2007, Chris Warren (Chris.Warren@williams.edu) and 
Paul Karabinos (Paul.M.Karabinos@williams.edu)

This program is licensed under the terms of the Williams College Software 
License as published by Williams College; either version 1 of the License, or 
(at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
PARTICULAR PURPOSE. See the Williams College Software License for more details.

You should have received a copy of the Williams College Software License along 
with this program; if not, write to Williams College Office for Information 
Technology, 22 Lab Campus Drive, Williamstown, MA 01267 USA
 *
 */

package edu.williams.geoshear;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.geom.AffineTransform;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.ListIterator;
import javax.swing.JPanel;

/**
 *
 * @author cwarren
 */
public abstract class PebblePlotterCartesian extends PebblePlotter
{
    private Line2D.Double xAxis;
    private Line2D.Double yAxis;
    
    private Line2D.Double harmonicMeanLine;
    private Line2D.Double vectorMeanLine;

    private int xAxisSize;
    private int yAxisSize;
    
    private String xLabel = "";
    private String xLabelOrig = "";
    private String yLabel = "";
    private String xMinValLabel = "";
    private String xMaxValLabel = "";
    private String yMinValLabel = "";
    private String yMaxValLabel = "";
    
    private double xMaxValAuto = 0.0;
    private double xMaxValManual = 0.0;
    private double xMinVal = 0.0;
    private double yMinVal = 0.0;
    private double yMaxVal = 0.0;
    
    private boolean manualXMax = false;
    private boolean showRFOnPebble = false;
        
    private int xInsetLeft = 16;
    private int xInsetRight = 4;
    private int yInsetTop  = 20;
    private int yInsetBottom  = 16;
    
    /**
     * Creates a new instance of PebblePlotterCartesian
     */
    public PebblePlotterCartesian ()
    {
        this.setMaximumSize (new Dimension(20000,10000));
        this.setMinimumSize (new Dimension(0,0));
        this.setPreferredSize (new Dimension(800,600));
        this.setSize (100,100);
    }
    
    public void setUpPlot ()
    {
        this.setUpInsets ();
        this.setUpAxes ();
        this.setUpPlotFrame ();
    }
    
    private void setUpInsets ()
    {
        this.xInsetLeft = this.getTextAllowance();
        this.xInsetRight = this.getGeneralInset();
        this.yInsetTop = this.getGeneralInset() + this.getTextAllowance();
        this.yInsetBottom = this.getTextAllowance();
    }
    
    public void setUpAxes ()
    {
        this.setXAxisSize (this.getBounds ().width - (this.xInsetLeft + this.xInsetRight));
        this.setYAxisSize (this.getBounds ().height - (this.yInsetBottom + this.yInsetTop));
        
        setPlotOrigin (new Point (this.xInsetLeft, this.yInsetTop + getYAxisSize ()));
        
        setXAxis (new Line2D.Double (getPlotOrigin ().x, getPlotOrigin ().y, this.xInsetLeft + getXAxisSize (), this.yInsetTop + getYAxisSize ()));
        setYAxis (new Line2D.Double (getPlotOrigin ().x, getPlotOrigin ().y, this.xInsetLeft, this.yInsetTop));
    }
    
    //**************************************************************************
    // OVERRIDE METHODS (and associated support)
    //**************************************************************************
    
    public void paintComponent (Graphics g)
    {
        super.paintComponent (g); //paints the background and image
        
        Graphics2D g2D = (Graphics2D) g;
        
        g2D.drawImage (this.getPlotFrame (),null,0,0);
        
        // 2009/05/20 - Question to past self, why does this need to be done in two seperate passes?
        // 2009/05/20 - looks like it may be because selected pebbles often show more info, which shouldn't be hidden by non-selected points...?
        
        // 2009/05/25 - significant changed here to allow pebblelist and chartpebblelist to correspond:
        //   - changing from iterators to for loops to keep lists in step
        
/*        // first the non-selected pebbles
        ListIterator pli = getPebbleList ().listIterator ();
        while (pli.hasNext ())
        {
            Pebble peb = (Pebble)(pli.next ());
            if (! peb.isSelected ())
            {
                plotPebble (g2D,peb);
            }
        }
        
        // then the selected ones
        pli = getPebbleList ().listIterator ();
        while (pli.hasNext ())
        {
            Pebble peb = (Pebble)(pli.next ());
            if (peb.isSelected ())
            {
                plotPebble (g2D,peb);
            }
        }        
 */
        // first the non-selected pebbles
        ArrayList selPebs = new ArrayList();
        for (int i = pebbleList.size ()-1; i >= 0; i--)
        {
            Pebble peb = (Pebble)(pebbleList.get (i));
            if (! peb.isSelected ())
            {
                plotPebble (g2D,peb
                               ,(Pebble)(chartPebbleList.get (i)));
            } 
            else {
                selPebs.add(i);
            }
        }
        for (int si = selPebs.size ()-1; si >= 0; si--)
        {
            plotPebble (g2D,(Pebble)(pebbleList.get ((Integer)(selPebs.get(si))))
                           ,(Pebble)(chartPebbleList.get ((Integer)(selPebs.get(si)))));
        }
        
        if (this.isShowHarmonics() && (this.getChartPebbleList().size() > 0))
        {
            // get harmonics
            this.setMeans();
            
            // show lines
            this.paintMeans(g2D);
        }

        // DEBUGGING
        //g2D.drawString ("origin: "+plotOrigin.x+","+plotOrigin.y,20,20);
    }
    
    
    protected void addLine (Graphics2D g2D, Line2D.Double l2D)
    {
        g2D.drawLine ((int)(l2D.x1),(int)(l2D.y1),(int)(l2D.x2),(int)(l2D.y2));
    }
    
    // NOTE: this is over-ridden in sub-classes!
    protected void plotPebble (Graphics2D g2D, Pebble peb, Pebble chartPeb)
    {
        int xOffset = 20;
        int yOffset = 30;
        
        Color origColor = g2D.getColor ();
        g2D.setColor (peb.getPebbleColor ());
        g2D.drawRect (this.getPlotOrigin ().x + xOffset,this.getPlotOrigin ().y - yOffset,1,1);
        g2D.setColor (origColor);
    }
    
    protected void setUpPlotFrame ()
    {
        int xInit = 100;
        int yInit = 100;
        if (this.getBounds ().width > 0)
        {
            xInit = this.getBounds ().width;
        }
        if (this.getBounds ().height > 0)
        {
            yInit = this.getBounds ().height;
        }
        this.setPlotFrame (new BufferedImage (xInit, yInit, BufferedImage.TYPE_INT_RGB));
        Graphics2D g2D = getPlotFrame ().createGraphics ();
        
        g2D.setColor (Color.WHITE);
        g2D.fillRect (0,0,this.getPreferredSize ().width, this.getPreferredSize ().height);
        
        Font origFont = g2D.getFont ();
        Color origColor = g2D.getColor ();
        g2D.setColor (Color.BLACK);
        
        g2D.setFont (getPlotTitleFont ());
        this.drawPlotTitle (g2D);
        
        g2D.setFont (getPlotFont ());
        this.drawYMaxVal (g2D);
        this.drawYLabel (g2D);
        this.drawYMinVal (g2D);
        this.drawXMinVal (g2D);
        this.drawXLabel (g2D);
        this.drawXMaxVal (g2D);
        g2D.setFont (origFont);
        
        this.addLine (g2D, this.getXAxis ());
        this.addLine (g2D, this.getYAxis ());
        
        g2D.setColor (origColor);
    }
    
    protected void drawPlotTitle (Graphics2D g2D)
    {
        if (this.title.length () > 0)
        {
            g2D.drawString (this.title,
                    this.getPlotOrigin ().x + (this.getXAxisSize ()/2) - ((int)(g2D.getFontMetrics ().getStringBounds (this.title,(Graphics)g2D).getWidth ())/2),
                    1 + g2D.getFontMetrics ().getHeight ());
        }
    }
    
    protected void drawYMaxVal (Graphics2D g2D)
    {
        if (this.yMaxValLabel.length () > 0)
        {
            Rectangle2D bounds = g2D.getFontMetrics ().getStringBounds (this.yMaxValLabel,(Graphics)g2D);
            BufferedImage textImage = new BufferedImage ((int)bounds.getWidth (),(int)bounds.getHeight ()+3,BufferedImage.TYPE_INT_RGB);
            Graphics2D textG2D = textImage.createGraphics ();
            textG2D.setColor (Color.WHITE);
            textG2D.fillRect (0,0,(int)bounds.getWidth (),(int)bounds.getHeight ()+3);
            textG2D.setColor (Color.BLACK);
            textG2D.setFont (this.getPlotFont ());
            textG2D.drawString (this.yMaxValLabel,0,(int)bounds.getHeight ());
            
            g2D.drawImage (textImage,
                    new AffineTransformOp (AffineTransform.getRotateInstance (Math.PI/2.0,0,(int)bounds.getHeight ()),AffineTransformOp.TYPE_NEAREST_NEIGHBOR),
this.getGeneralInset(),                     this.yInsetTop-(this.getPlotFont ().getSize ()+1));
        }
    }
    
    protected void drawYLabel (Graphics2D g2D)
    {
        if (this.yLabel.length () > 0)
        {
            Rectangle2D bounds = g2D.getFontMetrics ().getStringBounds (this.yLabel,(Graphics)g2D);
            BufferedImage textImage = new BufferedImage ((int)bounds.getWidth (),(int)bounds.getHeight ()+3,BufferedImage.TYPE_INT_RGB);
            Graphics2D textG2D = textImage.createGraphics ();
            textG2D.setColor (Color.WHITE);
            textG2D.fillRect (0,0,(int)bounds.getWidth (),(int)bounds.getHeight ()+3);
            textG2D.setColor (Color.BLACK);
            textG2D.setFont (this.getPlotFont ());
            textG2D.drawString (this.yLabel,0,(int)bounds.getHeight ());
            
            g2D.drawImage (textImage,
                    new AffineTransformOp (AffineTransform.getRotateInstance (Math.PI/2.0,0,(int)bounds.getHeight ()),AffineTransformOp.TYPE_NEAREST_NEIGHBOR),
this.getGeneralInset(),                     this.yInsetTop + this.yAxisSize/2 - (this.getPlotFont ().getSize ()+1) -(int)(bounds.getWidth ()/2.0));
        }
    }
    
    protected void drawYMinVal (Graphics2D g2D)
    {
        if (this.yMinValLabel.length () > 0)
        {
            Rectangle2D bounds = g2D.getFontMetrics ().getStringBounds (this.yMinValLabel,(Graphics)g2D);
            BufferedImage textImage = new BufferedImage ((int)bounds.getWidth (),(int)bounds.getHeight ()+3,BufferedImage.TYPE_INT_RGB);
            Graphics2D textG2D = textImage.createGraphics ();
            textG2D.setColor (Color.WHITE);
            textG2D.fillRect (0,0,(int)bounds.getWidth (),(int)bounds.getHeight ()+3);
            textG2D.setColor (Color.BLACK);
            textG2D.setFont (this.getPlotFont ());
            textG2D.drawString (this.yMinValLabel,0,(int)bounds.getHeight ());
            
            g2D.drawImage (textImage,
                    new AffineTransformOp (AffineTransform.getRotateInstance (Math.PI/2.0,0,(int)bounds.getHeight ()),AffineTransformOp.TYPE_NEAREST_NEIGHBOR),
this.getGeneralInset(),                     this.yInsetTop + this.yAxisSize - (this.getPlotFont ().getSize ()+1) - (int)(bounds.getWidth ()));
        }
    }
    
    protected void drawXMinVal (Graphics2D g2D)
    {
        if (this.xMinValLabel.length () > 0)
        {
            g2D.drawString (this.xMinValLabel,
                    this.getPlotOrigin ().x,
                    this.getPlotOrigin ().y + 1 + g2D.getFontMetrics ().getHeight ());
        }
    }
    
    protected void drawXLabel (Graphics2D g2D)
    {
        if (this.xLabel.length () > 0)
        {
            g2D.drawString (this.xLabel,
                    this.getPlotOrigin ().x + (this.getXAxisSize ()/2) - ((int)(g2D.getFontMetrics ().getStringBounds (this.xLabel,(Graphics)g2D).getWidth ())/2),
                    this.getPlotOrigin ().y + 1 + g2D.getFontMetrics ().getHeight ());
        }
    }
    
    protected void drawXMaxVal (Graphics2D g2D)
    {
        if (this.xMaxValLabel.length () > 0)
        {
            g2D.drawString (this.xMaxValLabel,
                    this.getPlotOrigin ().x + this.getXAxisSize () - (int)(g2D.getFontMetrics ().getStringBounds (this.xMaxValLabel,(Graphics)g2D).getWidth ()),
                    this.getPlotOrigin ().y + 1 + g2D.getFontMetrics ().getHeight ());
        }
    }
    
    //**************************************************************************
    // GET/SET METHODS
    //**************************************************************************
    
    
    public Line2D.Double getXAxis ()
    {
        return xAxis;
    }
    
    public void setXAxis (Line2D.Double xAxis)
    {
        this.xAxis = xAxis;
    }
    
    public Line2D.Double getYAxis ()
    {
        return yAxis;
    }
    
    public void setYAxis (Line2D.Double yAxis)
    {
        this.yAxis = yAxis;
    }
    
    public int getXAxisSize ()
    {
        return xAxisSize;
    }
    
    public void setXAxisSize (int xAxisSize)
    {
        this.xAxisSize = xAxisSize;
    }
    
    public int getYAxisSize ()
    {
        return yAxisSize;
    }
    
    public void setYAxisSize (int yAxisSize)
    {
        this.yAxisSize = yAxisSize;
    }
    
    public String getXLabel ()
    {
        return xLabel;
    }
    
    public void setXLabel (String xLabel)
    {
        this.xLabel = xLabel;
        if (this.getXLabelOrig().equals (""))
        {
            this.setXLabelOrig(xLabel);
        }
        this.setUpPlotFrame ();
    }
    
    public String getYLabel ()
    {
        return yLabel;
    }
    
    public void setYLabel (String yLabel)
    {
        this.yLabel = yLabel;
        this.setUpPlotFrame ();
    }
    
    public String getXMinValLabel ()
    {
        return xMinValLabel;
    }
    
    public void setXMinValLabel (String xMinVal)
    {
        this.xMinValLabel = xMinVal;
        this.setUpPlotFrame ();
    }
    
    public String getXMaxValLabel ()
    {
        return xMaxValLabel;
    }
    
    public void setXMaxValLabel (String xMaxVal)
    {
        /*if (xMaxVal.indexOf (".") > 0)
        {
            int truncLength = xMaxVal.length () - xMaxVal.indexOf (".");
            if (truncLength > 4)
            {
                truncLength = 4;
            }
            xMaxVal = xMaxVal.substring (0,xMaxVal.indexOf (".")+truncLength);
        }*/
        this.xMaxValLabel = Utils.truncForDisplay (xMaxVal);
        this.setUpPlotFrame ();
    }
    
    public String getYMinValLabel ()
    {
        return yMinValLabel;
    }
    
    public void setYMinValLabel (String yMinVal)
    {
        this.yMinValLabel = yMinVal;
        this.setUpPlotFrame ();
    }
    
    public String getYMaxValLabel ()
    {
        return yMaxValLabel;
    }
    
    public void setYMaxValLabel (String yMaxVal)
    {
        this.yMaxValLabel = yMaxVal;
        this.setUpPlotFrame ();
    }
        
    public double getXMaxValAuto ()
    {
        return xMaxValAuto;
    }
    
    public void setXMaxValAuto (double xMaxVal)
    {
        this.xMaxValAuto = xMaxVal;
    }
    
    public double getXMinVal ()
    {
        return xMinVal;
    }
    
    public void setXMinVal (double xMinVal)
    {
        this.xMinVal = xMinVal;
    }
    
    public double getYMinVal ()
    {
        return yMinVal;
    }
    
    public void setYMinVal (double yMinVal)
    {
        this.yMinVal = yMinVal;
    }
    
    public double getYMaxVal ()
    {
        return yMaxVal;
    }
    
    public void setYMaxVal (double yMaxVal)
    {
        this.yMaxVal = yMaxVal;
    }
            
    public int getXInsetLeft ()
    {
        return xInsetLeft;
    }

    public int getXInsetRight ()
    {
        return xInsetRight;
    }

    public int getYInsetTop ()
    {
        return yInsetTop;
    }

    public int getYInsetBottom ()
    {
        return yInsetBottom;
    }

    public boolean isManualXMax ()
    {
        return manualXMax;
    }

    public void setManualXMax (boolean manualXMax)
    {
        this.manualXMax = manualXMax;
    }

    /**
     * Getter for property xMaxValManual.
     * @return Value of property xMaxValManual.
     */
    public double getXMaxValManual ()
    {
        return this.xMaxValManual;
    }

    /**
     * Setter for property xMaxValManual.
     * @param xMaxValManual New value of property xMaxValManual.
     */
    public void setXMaxValManual (double xMaxValManual)
    {
        this.xMaxValManual = xMaxValManual;
    }

    public boolean isShowRFOnPebble ()
    {
        return showRFOnPebble;
    }

    public void setShowRFOnPebble (boolean showRFOnPebble)
    {
        this.showRFOnPebble = showRFOnPebble;
    }

    public String getXLabelOrig ()
    {
        return xLabelOrig;
    }

    public void setXLabelOrig (String xLabelOrig)
    {
        this.xLabelOrig = xLabelOrig;
    }

    public Line2D.Double getHarmonicMeanLine() {
        return harmonicMeanLine;
    }

    public void setHarmonicMeanLine(Line2D.Double harmonicMeanLine) {
        this.harmonicMeanLine = harmonicMeanLine;
    }

    public Line2D.Double getVectorMeanLine() {
        return vectorMeanLine;
    }

    public void setVectorMeanLine(Line2D.Double vectorMeanLine) {
        this.vectorMeanLine = vectorMeanLine;
    }
}
