/*
 * PebblePlotterPolar.java
 *
 * Created on June 28, 2007, 2:36 PM
 *
GeoShear - A program to explore deformation of cross sections of stone
    
Copyright (C) September 2007, Chris Warren (Chris.Warren@williams.edu) and 
Paul Karabinos (Paul.M.Karabinos@williams.edu)

This program is licensed under the terms of the Williams College Software 
License as published by Williams College; either version 1 of the License, or 
(at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
PARTICULAR PURPOSE. See the Williams College Software License for more details.

You should have received a copy of the Williams College Software License along 
with this program; if not, write to Williams College Office for Information 
Technology, 22 Lab Campus Drive, Williamstown, MA 01267 USA
 *
 */

package edu.williams.geoshear;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.geom.AffineTransform;
import java.awt.geom.Line2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Point2D;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.ListIterator;

/**
 *
 * @author cwarren
 */
public abstract class PebblePlotterPolar extends PebblePlotter
{
    private String radiusLabel = "";
    private String angleLabel = "";
    
    private double radiusMax = 3.0;
    private double radiusIncrements = 0.5;
    private double radiusMaxDim = 0;
    private double radiusSmallDim = 0;
    private int angleIncrements = 15; // NOTE: will give strange results if not / 360
    
    private ArrayList radii;
    private ArrayList spokes;
    private ArrayList spokeLabelAnchors;
    
    private ArrayList scale; // first elt is line, others are ticks
    private ArrayList scaleLabels;
    private ArrayList scaleLabelAnchors;
    private int scalePositionOffset = 32;
    private int scaleLabelOffset = 28;
    private int scaleTickSize = 5;
    private int scaleAllowance = scalePositionOffset + scaleLabelOffset;

    /** Creates a new instance of PebblePlotterPolar */
    public PebblePlotterPolar ()
    {
        this.setMaximumSize (new Dimension(20000,10000));
        this.setMinimumSize (new Dimension(0,0));
        this.setPreferredSize (new Dimension(800,600));
        this.setSize (100,100); 
        this.setManualXMax (true);
    }
    
    public void setUpPlot ()
    {
       this.setUpAxes();
       this.setUpPlotFrame ();
    }
        
    public void setUpAxes ()
    {
        // origin is center of panel, adjusted for insets
        this.setPlotOrigin(new Point ((this.getBounds ().width / 2) + (scaleAllowance/2), (this.getBounds ().height / 2) + this.getTextAllowance()/2));
        
        // radii and scale
        this.setRadiusMaxDim(Math.min (this.getBounds ().width - this.getPlotOrigin().x - this.getTextAllowance() - this.getGeneralInset(),
                                      (this.getBounds ().height - this.getPlotOrigin().y) - this.getGeneralInset()- this.getTextAllowance()));
        int numRadii = (int)(this.getRadiusMax ()/this.getRadiusIncrements());
        //this.radiiDims = new ArrayList(numRadii);
        this.setRadii(new ArrayList ());
        this.setScale(new ArrayList ());
        this.setScaleLabels(new ArrayList ());
        this.setScaleLabelAnchors(new ArrayList ());
        double scaleX = this.getPlotOrigin().x - this.getRadiusMaxDim () - scalePositionOffset;
        this.getScale ().add (new Line2D.Double(scaleX,this.getPlotOrigin().y,scaleX,this.getPlotOrigin().y-this.getRadiusMaxDim ()));
        this.getScale ().add (new Line2D.Double(scaleX,this.getPlotOrigin().y,scaleX+5,this.getPlotOrigin().y));
        this.getScaleLabels ().add ("0");
        this.getScaleLabelAnchors ().add (new Point((int)(scaleX-scaleLabelOffset),getPlotOrigin().y));
        for (int r = 1; r <= numRadii; r++)
        {
            double radius = (double)(this.getRadiusMaxDim()) * (double)r/(double)(numRadii);
            if (r==1)
            {
                this.setRadiusSmallDim(radius);
            }
            this.getRadii().add (new Ellipse2D.Double(this.getPlotOrigin().x - radius,
                                                   this.getPlotOrigin().y - radius,
                                                   radius * 2,
                                                   radius * 2));
            this.getScale ().add (new Line2D.Double(scaleX,this.getPlotOrigin().y-radius,
                                                    scaleX+this.scaleTickSize,this.getPlotOrigin().y-radius));
            this.getScaleLabels ().add (Utils.truncTextDecimal (Double.toString (r*this.getRadiusIncrements()),2));
            this.getScaleLabelAnchors ().add (new Point((int)(scaleX-this.scaleLabelOffset),
                                                        (int)(this.getPlotOrigin().y-radius)));
        }
        
        // spokes
        int numSpokes = 360 / this.getAngleIncrements();
        this.setSpokes(new ArrayList());
        this.setSpokeLabelAnchors(new ArrayList ());
        for (int s = 0; s < numSpokes; s++)
        {
            int effectiveAngle = s * this.getAngleIncrements();
            double angle = Math.toRadians (effectiveAngle);
            double effectiveSmallRadius = this.getRadiusSmallDim ();
            if ( (effectiveAngle == 0)
                || (effectiveAngle == 90)
                || (effectiveAngle == 180)
                || (effectiveAngle == 270))
            {
                effectiveSmallRadius = 0;
            }
            double smallX = effectiveSmallRadius * Math.cos(angle) + this.getPlotOrigin ().x;
            double smallY = effectiveSmallRadius * Math.sin(angle) + this.getPlotOrigin ().y;
            double bigX = this.getRadiusMaxDim() * Math.cos(angle) + this.getPlotOrigin ().x;
            double bigY = this.getRadiusMaxDim() * Math.sin(angle) + this.getPlotOrigin ().y;
            this.getSpokes().add (new Line2D.Double(smallX,smallY,bigX,bigY));
            this.getSpokeLabelAnchors().add (new Point((int)((this.getRadiusMaxDim()+12) * Math.cos(angle) + this.getPlotOrigin ().x),
                                                       (int)((this.getRadiusMaxDim()+12) * Math.sin(angle) + this.getPlotOrigin ().y)));
        }
    }

    protected void setUpPlotFrame ()
    {
        int xInit = 100;
        int yInit = 100;
        if (this.getBounds ().width > 0)
        {
            xInit = this.getBounds ().width;
        }
        if (this.getBounds ().height > 0)
        {
            yInit = this.getBounds ().height;
        }
        this.setPlotFrame (new BufferedImage (xInit, yInit, BufferedImage.TYPE_INT_RGB));
        Graphics2D g2D = getPlotFrame ().createGraphics ();
        
        g2D.setColor (Color.WHITE);
        g2D.fillRect (0,0,this.getPreferredSize ().width, this.getPreferredSize ().height);
        
        Font origFont = g2D.getFont ();
        Color origColor = g2D.getColor ();
        g2D.setColor (Color.BLACK);
        
        g2D.setFont (getPlotTitleFont ());
        this.drawPlotTitle (g2D);
        
        // now the radii and spokes
        ListIterator itr = this.getRadii ().listIterator ();
        while (itr.hasNext ())
        {
            this.addEllipse (g2D,(Ellipse2D.Double)(itr.next ()));
        }
        itr = this.getSpokes ().listIterator ();
        while (itr.hasNext ())
        {
            this.addLine (g2D,(Line2D.Double)(itr.next ()));
        }
        
        // and the spoke labels
        itr = this.getSpokeLabelAnchors ().listIterator ();
        int spokeAngleIncrementer = 0;
        while (itr.hasNext ())
        {
            Point labelPoint = (Point)(itr.next ());
            String angleLabel = ""+spokeAngleIncrementer;
            int labelPosX = (int)labelPoint.getX () - (g2D.getFontMetrics ().stringWidth (angleLabel)/2);
            int labelPosY = (int)labelPoint.getY () + (g2D.getFontMetrics ().getHeight ()/3);
            g2D.drawString (angleLabel,labelPosX,labelPosY);
            spokeAngleIncrementer -= this.getAngleIncrements ();
            if (spokeAngleIncrementer <= -180)
            {
                spokeAngleIncrementer += 360;
            }
        }

        // scale - lines
        itr = this.getScale ().listIterator ();
        while (itr.hasNext ())
        {
            this.addLine (g2D,(Line2D.Double)(itr.next ()));
        }        
        
        // scale - labels
        itr = this.getScaleLabelAnchors ().listIterator ();
        int scaleLabelCounter = 0;
        while (itr.hasNext ())
        {
            Point p = (Point)(itr.next ());
            g2D.drawString (this.getScaleLabels ().get (scaleLabelCounter).toString (),
                                                        p.x,
                                                        p.y+(g2D.getFontMetrics ().getHeight ()/3));
            scaleLabelCounter++;
        }        

        g2D.setColor (origColor);
    }    

    protected void addLine (Graphics2D g2D, Line2D.Double l2D)
    {
        g2D.drawLine ((int)(l2D.x1),(int)(l2D.y1),(int)(l2D.x2),(int)(l2D.y2));
    }

    protected void addEllipse (Graphics2D g2D, Ellipse2D.Double e2D)
    {
        g2D.drawOval ((int)e2D.x, (int)e2D.y, (int)e2D.width, (int)e2D.height);
    }
    
    protected void drawPlotTitle (Graphics2D g2D)
    {
        if (this.title.length () > 0)
        {
            
            g2D.drawString (this.title,
                    this.getPlotOrigin ().x - (g2D.getFontMetrics ().stringWidth (this.title)/2),
                    1 + g2D.getFontMetrics ().getHeight ());
        }
    }
    
    public void paintComponent (Graphics g)
    {
        super.paintComponent (g); //paints the background and image
                
        Graphics2D g2D = (Graphics2D) g;
        
        g2D.drawImage (this.getPlotFrame (),null,0,0);

 /*       // first the non-selected pebbles
        ListIterator pli = getPebbleList ().listIterator ();
        while (pli.hasNext ())
        {
            Pebble peb = (Pebble)(pli.next ());
            if (! peb.isSelected ())
            {
                plotPebble (g2D,peb);
            }
        }
        
        // then the selected ones
        pli = getPebbleList ().listIterator ();
        while (pli.hasNext ())
        {
            Pebble peb = (Pebble)(pli.next ());
            if (peb.isSelected ())
            {
                plotPebble (g2D,peb);
            }
        }        
*/
        ArrayList selPebs = new ArrayList();
        for (int i = pebbleList.size ()-1; i >= 0; i--)
        {
            Pebble peb = (Pebble)(pebbleList.get (i));
            if (! peb.isSelected ())
            {
                plotPebble (g2D,peb
                               ,(Pebble)(chartPebbleList.get (i)));
            } 
            else {
                selPebs.add(i);
            }
        }
        for (int si = selPebs.size ()-1; si >= 0; si--)
        {
            plotPebble (g2D,(Pebble)(pebbleList.get ((Integer)(selPebs.get(si))))
                           ,(Pebble)(chartPebbleList.get ((Integer)(selPebs.get(si)))));
        }

        if (this.isShowHarmonics() && (this.getChartPebbleList().size() > 0))
        {
            // get harmonics
            this.setMeans();
            
            // show lines
            this.paintMeans(g2D);
        }
        
        // DEBUGGING
        //g2D.drawString ("origin: "+plotOrigin.x+","+plotOrigin.y,20,20);
    }
    
    protected void plotPebble (Graphics2D g2D, Pebble peb, Pebble chartPeb)
    {
        Color origColor = g2D.getColor ();
        g2D.setColor (peb.getPebbleColor ());
        double radial = (chartPeb.getRatioLongToShort () / this.getRadiusMax ()) * this.getRadiusMaxDim ();
        g2D.drawRect (this.getPlotOrigin ().x+(int)(Math.cos (chartPeb.getThetaRad ())*Math.log (radial)),
                      this.getPlotOrigin ().y+(int)(Math.sin (chartPeb.getThetaRad ())*Math.log (radial)),
                      1,1);
        g2D.setColor (origColor);
    }

    
    //**************************************************************************
    // GET/SET METHODS
    //**************************************************************************

    public ArrayList getPebbleList ()
    {
        return pebbleList;
    }

    public void setPebbleList (ArrayList pebbleList)
    {
        this.pebbleList = pebbleList;
    }

    public Point getPlotOrigin ()
    {
        return plotOrigin;
    }

    public void setPlotOrigin (Point plotOrigin)
    {
        this.plotOrigin = plotOrigin;
    }

    public String getTitle ()
    {
        return title;
    }

    public void setTitle (String title)
    {
        this.title = title;
    }

    public String getRadiusLabel ()
    {
        return radiusLabel;
    }

    public void setRadiusLabel (String radiusLabel)
    {
        this.radiusLabel = radiusLabel;
    }

    public String getAngleLabel ()
    {
        return angleLabel;
    }

    public void setAngleLabel (String angleLabel)
    {
        this.angleLabel = angleLabel;
    }

    public double getRadiusMax ()
    {
        return radiusMax;
    }

    public void setRadiusMax (double radiusMax)
    {
        this.radiusMax = radiusMax;
    }

    public double getRadiusIncrements ()
    {
        return radiusIncrements;
    }

    public void setRadiusIncrements (double radiusIncrements)
    {
        this.radiusIncrements = radiusIncrements;
    }

    public double getRadiusMaxDim ()
    {
        return radiusMaxDim;
    }

    public void setRadiusMaxDim (double radiusMaxDim)
    {
        this.radiusMaxDim = radiusMaxDim;
    }

    public double getRadiusSmallDim ()
    {
        return radiusSmallDim;
    }

    public void setRadiusSmallDim (double radiusSmallDim)
    {
        this.radiusSmallDim = radiusSmallDim;
    }

    public int getAngleIncrements ()
    {
        return angleIncrements;
    }

    public void setAngleIncrements (int angleIncrements)
    {
        this.angleIncrements = angleIncrements;
    }

    public ArrayList getRadii ()
    {
        return radii;
    }

    public void setRadii (ArrayList radii)
    {
        this.radii = radii;
    }

    public ArrayList getSpokes ()
    {
        return spokes;
    }

    public void setSpokes (ArrayList spokes)
    {
        this.spokes = spokes;
    }

    public ArrayList getSpokeLabelAnchors ()
    {
        return spokeLabelAnchors;
    }

    public void setSpokeLabelAnchors (ArrayList spokeLabelAnchors)
    {
        this.spokeLabelAnchors = spokeLabelAnchors;
    }
    
    public void setXMaxValAuto (double xMaxValAuto)
    {
    }

    public ArrayList getScale ()
    {
        return scale;
    }

    public void setScale (ArrayList scale)
    {
        this.scale = scale;
    }

    public ArrayList getScaleLabels ()
    {
        return scaleLabels;
    }

    public void setScaleLabels (ArrayList scaleLabels)
    {
        this.scaleLabels = scaleLabels;
    }

    public ArrayList getScaleLabelAnchors ()
    {
        return scaleLabelAnchors;
    }

    public void setScaleLabelAnchors (ArrayList scaleLabelAnchors)
    {
        this.scaleLabelAnchors = scaleLabelAnchors;
    }

}
