/*
 * PlotRfVsPhi.java
 *
 * Created on March 21, 2007, 11:56 AM
 *
GeoShear - A program to explore deformation of cross sections of stone
    
Copyright (C) September 2007, Chris Warren (Chris.Warren@williams.edu) and 
Paul Karabinos (Paul.M.Karabinos@williams.edu)

This program is licensed under the terms of the Williams College Software 
License as published by Williams College; either version 1 of the License, or 
(at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
PARTICULAR PURPOSE. See the Williams College Software License for more details.

You should have received a copy of the Williams College Software License along 
with this program; if not, write to Williams College Office for Information 
Technology, 22 Lab Campus Drive, Williamstown, MA 01267 USA
 *
 */

package edu.williams.geoshear;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;

/**
 *
 * @author cwarren
 */
public class PlotRfVsPhi extends PebblePlotterCartesian
{
    private boolean logScaleX = false;
    private double effectiveXMax;
    private int SNAPSHOT_DISTANCE = 32; // distance in pixels between snapshots
    /** Creates a new instance of PlotRfVsPhi */
    public PlotRfVsPhi ()
    {
        super ();
    }
    
    protected void setUpPlotFrame ()
    {
        super.setUpPlotFrame ();
        Graphics2D g2D = getPlotFrame ().createGraphics ();
        Point po = this.getPlotOrigin ();
        Color origColor = g2D.getColor ();
        g2D.setColor (Color.LIGHT_GRAY);
        g2D.drawLine (po.x+1,po.y-(this.getYAxisSize ()/2),po.x+this.getXAxisSize (),po.y-(this.getYAxisSize ()/2));
        g2D.setColor (origColor);
    }
    
    /**
     * Draws the pebble and any associated info in the graphics object
     * 
     * @param g2D draw on this
     * @param peb holds color and selection info
     * @param chartPeb hold non-scaled version of peb for consistent plotting
     */
    protected void plotPebble (Graphics2D g2D, Pebble peb, Pebble chartPeb)
    {
        int xPos;
        int yPos;
        
        this.effectiveXMax = Utils.truncForDisplay (this.getXMaxValAuto () * 1.2);
        if (this.isManualXMax ())
        {
            this.effectiveXMax = this.getXMaxValManual ();
        }
        
        if (this.isLogScaleX ())
        {
            this.setXMaxValLabel (Double.toString (effectiveXMax));
        }
        else
        {
            this.setXMaxValLabel (Double.toString (effectiveXMax));
        }
        xPos = this.getPlotOrigin ().x + this.getXOffsetForPebble(chartPeb);
        //int yOffset = (int)((double)(this.getYAxisSize ()) * ((chartPeb.getThetaRad () * -1.0)/Math.PI)) + this.getYAxisSize ()/2;
        yPos = this.getPlotOrigin ().y - this.getYOffsetForPebble(chartPeb);
                
        // this sections checks to see whether a snapshot should be taken (i.e. if dist from prev point is > SNAPSHOT_DISTANCE)
        int numSnaps = chartPeb.getSnapshots().size();
        if (numSnaps < 1)
        {
            chartPeb.takeSnapshot();
        } else
        {
            Pebble prevChartPeb = (Pebble)(chartPeb.getSnapshots().get(numSnaps - 1));
            int xPosPrev = this.getPlotOrigin ().x + this.getXOffsetForPebble(prevChartPeb);
            int yPosPrev = this.getPlotOrigin ().y - this.getYOffsetForPebble(prevChartPeb);
            // check dist
            //if ( ((xPosPrev - xPos)^2 + (yPosPrev - yPos)^2) > (this.SNAPSHOT_DISTANCE^2) )
            //if ( (Math.pow(xPosPrev - xPos,2) + Math.pow(yPosPrev - yPos,2)) > Math.pow(this.SNAPSHOT_DISTANCE,2) )
            //if ( ((xPosPrev - xPos)^2 + (yPosPrev - yPos)^2) > (0) )
            if (! Utils.pointsWithin(xPosPrev, yPosPrev, xPos, yPos, this.SNAPSHOT_DISTANCE))
            {
                chartPeb.takeSnapshot();
            }

            //DEBUGGING
            //String distInfo = "dist is "+Integer.toString((int)(Math.sqrt(Math.pow(xPosPrev - xPos,2) + Math.pow(yPosPrev - yPos,2))));
            //g2D.drawString (distInfo,xPos+3,yPos+5);
        }   
        
        // show trace, if applicable
        // For trace to show:
        //   scale must be fixed
        //   flag must be set
        if (this.isShowTrace())
        {
            Color origColor = g2D.getColor ();
            Pebble curPeb = peb;
            Color curColor = peb.getColor();
            for (int i = numSnaps-1; i >= 0; i--)
            {
                Pebble toPeb = (Pebble)(chartPeb.getSnapshots().get(i));
                int xPosCur = this.getPlotOrigin ().x + this.getXOffsetForPebble(curPeb);
                int yPosCur = this.getPlotOrigin ().y - this.getYOffsetForPebble(curPeb);
                int xPosTo = this.getPlotOrigin ().x + this.getXOffsetForPebble(toPeb);
                int yPosTo = this.getPlotOrigin ().y - this.getYOffsetForPebble(toPeb);
                g2D.setColor(curColor);
                 // skip line if too far apart
                if (Utils.pointsWithin(xPosCur, yPosCur, xPosTo, yPosTo, this.SNAPSHOT_DISTANCE * 3))
                {
                    // solid line
                    // g2D.drawLine(xPosCur, yPosCur, xPosTo, yPosTo);
                    // dashed trace line
                    g2D.drawLine((xPosTo + xPosCur)/2, (yPosTo + yPosCur)/2, xPosTo, yPosTo);
                } 
                else 
                {
                    chartPeb.takeSnapshot();
                }
                
                // DEBUGGING
                /*
                Pebble prevChartPeb = (Pebble)(chartPeb.getSnapshots().get(numSnaps - 1));
                int xPosPrev = this.getPlotOrigin ().x + this.getXOffsetForPebble(prevChartPeb);
                int yPosPrev = this.getPlotOrigin ().y - this.getYOffsetForPebble(prevChartPeb);
                //int dist = ((xPosTo - xPosCur)^2 + (yPosTo - yPosCur)^2);
                int dist = (int)(Math.pow(xPosTo - xPosCur,2));
                //String curInfo = "cur is "+Integer.toString(dist)+" away from to";
                //String toInfo = "to";
                String curInfo = "cur is ("+Integer.toString(xPosCur)+","+Integer.toString(yPosCur)+")";
                String toInfo = "to is ("+Integer.toString(xPosTo)+","+Integer.toString(yPosTo)+")";
                String distInfo = "dist is "+Integer.toString(dist);
                g2D.drawString (curInfo,xPosCur+1,yPosCur+this.getMarkSize ());
                g2D.drawString (toInfo,xPosTo+1,yPosTo+this.getMarkSize ()+15);
                g2D.drawString (distInfo,xPosCur+1,yPosCur+this.getMarkSize ()-15);
                */
                
                curColor = Utils.getLighterColor(curColor, .02);
                curPeb = toPeb;
            }
            g2D.setColor(origColor);
        }

        // offset main position for mark size
        xPos -= (this.getMarkSize ()/2);
        yPos -= (this.getMarkSize ()/2);
                
        // show selection highlighting
        if (peb.isSelected ())
        {
            Color origColor = g2D.getColor ();
            g2D.setColor (Pebble.SELECTION_PLOT_COLOR);
            
            int markX = xPos-2;
            int markY = yPos-2;
            int markDim = this.getMarkSize ()+4;

            if (this.getMarkShape () == PebblePlotterCartesian.MARK_RECT)
            {
                g2D.fillRect (markX,markY,markDim,markDim);
                //g2D.drawRect (markX,markY,markDim,markDim);
            }
            else if (this.getMarkShape () == PebblePlotterCartesian.MARK_CIRCLE)
            {
                g2D.fillOval (markX,markY,markDim,markDim);
                //g2D.drawOval (markX,markY,markDim,markDim);
            }
            else
            { // defaults to rects
                g2D.fillRect (markX,markY,markDim,markDim);
                //g2D.drawRect (markX,markY,markDim,markDim);
            }
            
            g2D.setColor (origColor);
        }
        
        Color origColor = g2D.getColor ();
        g2D.setColor (peb.getPebbleColor ());
        if (this.getMarkShape () == PebblePlotterCartesian.MARK_RECT)
        {
            if (this.isMarkFill ())
            {
                g2D.fillRect (xPos,yPos,this.getMarkSize (),this.getMarkSize ());
            }
            else
            {
                g2D.drawRect (xPos,yPos,this.getMarkSize (),this.getMarkSize ());
            }
        }
        else if (this.getMarkShape () == PebblePlotterCartesian.MARK_CIRCLE)
        {
            if (this.isMarkFill ())
            {
                g2D.fillOval (xPos,yPos,this.getMarkSize (),this.getMarkSize ());
            }
            else
            {
                g2D.drawOval (xPos,yPos,this.getMarkSize (),this.getMarkSize ());
            }
        }
        else
        { // defaults to rects
            if (this.isMarkFill ())
            {
                g2D.fillRect (xPos,yPos,this.getMarkSize (),this.getMarkSize ());
            }
            else
            {
                g2D.drawRect (xPos,yPos,this.getMarkSize (),this.getMarkSize ());
            }
        }
        
        // NOTE: info only shows on selected pebbles
        if (this.isShowRFOnPebble () && peb.isSelected ())
        {
            g2D.setColor (Color.BLACK);
            String rFText = Utils.truncForDisplay (Double.toString (chartPeb.getRatioLongToShort ()));            
/*            rfText = Utils.truncTextDecimal (rfText,3);
            if (rFText.indexOf (".") > 0)
            {
                int truncLength = rFText.length () - rFText.indexOf (".");
                if (truncLength > 4)
                {
                    truncLength = 4;
                }
                rFText = rFText.substring (0,rFText.indexOf (".")+truncLength);
            }
  */          
            String longAxisText = Integer.toString (chartPeb.getLongAxisLength ());            
            String shortAxisText = Integer.toString (chartPeb.getShortAxisLength ());            

            String phiText = Utils.truncForDisplay (Double.toString (chartPeb.getThetaRad ()));
/*            if (phiText.indexOf (".") > 0)
            {
                int truncLength = phiText.length () - phiText.indexOf (".");
                if (truncLength > 4)
                {
                    truncLength = 4;
                }
                phiText = phiText.substring (0,phiText.indexOf (".")+truncLength);
            }
  */          
            //String pebbleInfo = "phi="+phiText+"; rf="+rFText+"=("+longAxisText+"/"+shortAxisText+")";
            String pebbleInfo = "x="+rFText+" y="+phiText;
            g2D.drawString (pebbleInfo,xPos+this.getMarkSize ()+ 3,yPos+this.getMarkSize ());
        }
        g2D.setColor (origColor);
    }
    
    protected void paintMeans(Graphics2D g2D)
    {
        //g2D.drawLine(x1, y1, x2, y2);

        int harmonicMeanPlacement = getXOffsetFromRFValue(this.harmonicMean);
        int vectorMeanPlacement = getYOffsetFromPhiValue(this.vectorMean);
        
        // CSW debugging
        //g2D.drawString("harmonicMean=" + this.harmonicMean + ", vectorMean=" + this.vectorMean, 30, 30);
        //Pebble p1 = (Pebble)(this.getChartPebbleList().get(0));
        //g2D.drawString("p1.x(RF)=" + p1.getRatioLongToShort() + ", p1.y(theta)=" + p1.getThetaRad(), 30, 55);
        
        Color origColor = g2D.getColor();
        g2D.setColor (Color.GRAY);
        // harmonic mean, vert
        //g2D.drawLine(getPlotOrigin ().x + harmonicMeanPlacement, getPlotOrigin ().y - getYAxisSize(), getPlotOrigin ().x + harmonicMeanPlacement, getPlotOrigin ().y);
        g2D.fillRect(getPlotOrigin ().x + harmonicMeanPlacement - 1,
                      getPlotOrigin ().y - getYAxisSize(),
                      2,
                      getYAxisSize());
        
        // vector mean, horiz
        //g2D.drawLine(getPlotOrigin ().x,
        //             getPlotOrigin ().y - vectorMeanPlacement,
        //             getPlotOrigin ().x+getXAxisSize (),
        //             getPlotOrigin ().y - vectorMeanPlacement);
        g2D.fillRect(getPlotOrigin ().x,
                     getPlotOrigin ().y - vectorMeanPlacement-1,
                     getXAxisSize (),
                     2);
        
        g2D.setColor (origColor);
    }

    private int getXOffsetForPebble(Pebble p)
    {
     /*   if (this.isLogScaleX ())
        {
            return (int)((double)(this.getXAxisSize ()) * (Math.log10 (p.getRatioLongToShort ()) / (Math.log10 (this.effectiveXMax)) ));
        }
        else
        {
            return (int)((double)(this.getXAxisSize ()) * (p.getRatioLongToShort () / this.effectiveXMax));
        } */
        return getXOffsetFromRFValue(p.getRatioLongToShort ());
    }

    private int getXOffsetFromRFValue(double rf)
    {
        if (this.isLogScaleX ())
        {
            return (int)((double)(this.getXAxisSize ()) * (Math.log10 (rf) / (Math.log10 (this.effectiveXMax)) ));
        }
        else
        {
            return (int)((double)(this.getXAxisSize ()) * (rf / this.effectiveXMax));
        }
    }
    
    private int getYOffsetForPebble(Pebble p)
    {
        //return (int)((double)(this.getYAxisSize ()) * ((p.getThetaRad () * -1.0)/Math.PI)) + this.getYAxisSize ()/2;
        return getYOffsetFromPhiValue(p.getThetaRad ());
    }

    private int getYOffsetFromPhiValue(double phi)
    {
        return (int)((double)(this.getYAxisSize ()) * ((phi * -1.0)/Math.PI)) + this.getYAxisSize ()/2;
    }
    
    public boolean isLogScaleX ()
    {
        return logScaleX;
    }
    
    public void setLogScaleX (boolean logScaleX)
    {
        this.logScaleX = logScaleX;
        if (logScaleX)
        {
            this.setXLabel ("log10 of "+this.getXLabelOrig ());
        } else
        {
            this.setXLabel (this.getXLabelOrig ());
        }
    }
    
}
