/*
 * StrainPebble.java
 *
 * Created on June 5, 2007, 2:39 PM
 *
GeoShear - A program to explore deformation of cross sections of stone
    
Copyright (C) September 2007, Chris Warren (Chris.Warren@williams.edu) and 
Paul Karabinos (Paul.M.Karabinos@williams.edu)

This program is licensed under the terms of the Williams College Software 
License as published by Williams College; either version 1 of the License, or 
(at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
PARTICULAR PURPOSE. See the Williams College Software License for more details.

You should have received a copy of the Williams College Software License along 
with this program; if not, write to Williams College Office for Information 
Technology, 22 Lab Campus Drive, Williamstown, MA 01267 USA
 *
 */

package edu.williams.geoshear;

import java.awt.Color;
import java.awt.geom.AffineTransform;

/**
 *
 * @author cwarren
 */
public class StrainPebble extends Pebble
{
    private MainWindow watcher;
    
    /** Creates a new instance of StrainPebble */
    public StrainPebble ()
    {
        super();
    }

    public StrainPebble (int x, int y, int longAxis, int shortAxis, double thetaRad)
    {
        super (x,y,longAxis,shortAxis,thetaRad, Color.BLACK);
    }

    public StrainPebble (int x, int y, int longAxis, int shortAxis, int thetaDeg)
    {
        super (x,y,longAxis,shortAxis,Math.toRadians (thetaDeg), Color.BLACK);
    }
    
    // NOTE: degrees are specified only in int - given our resolution I don't
    // care about less than 1/360 of an arc (if you do, then use the radians)
    public StrainPebble (int x, int y, int longAxis, int shortAxis, int thetaDeg, Color c)
    {
        super (x,y,longAxis,shortAxis,Math.toRadians (thetaDeg), c);
    }
    
    public StrainPebble (int x, int y, int longAxis, int shortAxis, double thetaRad, Color c)
    {
        super (x, y, longAxis, shortAxis, thetaRad, c);
    }

    public StrainPebble (String s)
    {
        super (s);
    }
    
    /*************************************************************
     * OVERRIDES
     *************************************************************
     */    

    public StrainPebble clone()
    {
        return new StrainPebble(this.serialize ());
    }
    
    public void transformMe (AffineTransform transformer)
    {
        this.copyOrigKeysToActiveKeys ();
        
        if (transformer == null)
        {
            transformer = this.shapeCopier;
        }
        
        this.pebbleShape = transformer.createTransformedShape (this.basePebbleShape);
        transformer.transform (this.center, this.center);
        
        this.determineLongAndShortAxis ();
        if (watcher != null)
        {
            watcher.updateStrainInfoFromPebble (this);
        }
    }
    
    
    public MainWindow getWatcher ()
    {
        return watcher;
    }

    public void setWatcher (MainWindow watcher)
    {
        this.watcher = watcher;
    }
    
}
